//
//  ConcentrationTest.swift
//  ConcentrationTest
//
//  Created by Luciano Bononi on 07/04/18.
//  Copyright © 2018 Luciano Bononi. All rights reserved.
//

import Foundation

class ConcentrationTest {
    private(set) var cards = [Card]()
    
   private var indexOfOneAndOnlyFaceUpCard: Int? {
        get {
            // example fo closure here to make more elegant indexOfOneAndOnlyFaceUpCard alternative to commented code below
            let faceUpCardIndices = cards.indices.filter { cards[$0].isFaceUp }
            return faceUpCardIndices.count == 1 ? faceUpCardIndices.first : nil
   
// uncomment this line and comment the two above after having implemented the Collections extension
//            return cards.indices.filter { cards[$0].isFaceUp }.oneAndOnly
            
//            var foundIndex: Int?
//            for index in cards.indices {
//                if cards[index].isFaceUp {
//                    if foundIndex == nil {
//                        foundIndex = index
//                    } else {
//                        return nil
//                    }
//                }
//            }
//            return foundIndex
        }
        set(newValue) {
            for index in cards.indices {
                cards[index].isFaceUp = (index == newValue)
            }
        }
    }
    
    func chooseCard(at index: Int) {
        assert(cards.indices.contains(index), "Concentration.chooseCard(at: \(index)): chosen index is not in the cards")
        if !cards[index].isMatched {
            if let matchIndex = indexOfOneAndOnlyFaceUpCard, matchIndex != index {
                //check if cards match
                if cards[matchIndex].identifier == cards[index].identifier {
                    cards[matchIndex].isMatched = true
                    cards[index].isMatched = true
                }
                cards[index].isFaceUp = true
                //               indexOfOneAndOnlyFaceUpCard = nil //NB set to nil since it is defined as Optional, but not needed after computed property was created
            } else {
                // either no cards or 2 cards are face Up, first put all cards face down
                //                for flipDownIndex in cards.indices {
                //                    cards[flipDownIndex].isFaceUp = false
                //                }
                //                cards[index].isFaceUp = true
                indexOfOneAndOnlyFaceUpCard = index
            }
        }
    }
    
    func reset() {
        // alternative function for resetting the game, to do as homework
    }
    
    init(numberOfPairsOfCards: Int) {
        assert(numberOfPairsOfCards > 0, "Concentration.init(\(numberOfPairsOfCards)): you must have at least one pair of cards")
        for _ in 1...numberOfPairsOfCards {
        let card = Card()
        cards += [card, card]
        }
        
        // TODO: shuffle cards
         for swapFromIndex in stride(from: 2, through: 2*(numberOfPairsOfCards - 1), by: 2) {
            let swapToIndex = Int(arc4random_uniform(UInt32(numberOfPairsOfCards))) * 2
            let temp = cards[swapFromIndex].identifier
            cards[swapFromIndex].identifier = cards[swapToIndex].identifier
            cards[swapToIndex].identifier = temp
         }
    }
}

// even one more alternative for implementation extending the Collection type with var oneAndOnly!
extension Collection {
    var oneAndOnly: Element? {
        return count == 1 ? first : nil
    }
    
}
